/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* ProduceAnimGif, Miklos Reiter, 05-May-2001
 * modified from TestSphereCollision, Miklos Reiter, 26 March, 2001
 * modified from TestPhysics, James Kermode 22-Mar-2001
 * modified from TestAll_miklos, Miklos Reiter, March 21 2001
 *
 * Test program to test the TestSphereCollision, the OutputProjection and associated classes.
 *
 * @author Miklos Reiter
 * @version 05-May-2001
 */

import java.util.ListIterator;

import noope.core.*;
import noope.input.*;
import noope.laws.Law;
import noope.output.OutputProjection;
import noope.entities.Entity;

public class ProduceAnimGif
{
    public static void main(String[] argv) throws BRLoadingException
    {
	double deltat = 0.001;
	
	String s = "0.1"; // the sampling period
	
	// Set up an ArrayBlockReader to load physics from
	BlockReader[] lawsBlocks = {newGravBr(), newSphereCollBr(),
				    // projection onto the x-y plane
				    newOutputProjectionBr("O1", s, "(0,0,1)", "-4", "4", "-4", 
							  "4", "300", "300", "anim.gif")};
	
	
        BlockReader[] entitiesBlocks = {newEntity1Br(), newEntity2Br(), newEntity3Br()};
	BlockReader lawsBr = newLawsBr(lawsBlocks);
	BlockReader entitiesBr = newEntitiesBr(entitiesBlocks);
	BlockReader physicsBr = newPhysicsBr(lawsBr, entitiesBr);

	// Construct physics from this ArrayBlockReader
	Physics physics = new Physics(physicsBr);

	// Run simulation for 10 seconds
	for (int i = 0; i < 10000; i++)
	    {
		physics.step(deltat);
	    }

	// among other things, this will save the gif file
	physics.dispose();

	// now exit
	System.exit(0);
    }

    
    // *** Miklos' ArrayBlockReader test functions:
	
    private static BlockReader newGravBr() // creates an ArrayBlockReader defining a Gravity law
    {
	String header = "law", parameter = "Gravity"; 
	String[][] entries = 
	{{"name", "Gravity"},
	 {"G", "1"}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    private static BlockReader newEntity1Br() // creates an ArrayBlockReader defining an Entity
    {
	String header = "entity", parameter = ""; // these aren't really needed, but every BlockReader has to have
	// them
	String[][] entries = 
	{{"name", "Entity1"},
	 {"position", "(1, 0, 0)"},
	 {"velocity", "(0,0,0)"},
	 {"inertialMass", "1"}
	};
	BlockReader[] subblocks = new BlockReader[]
	{
	    newSphereCollPropBr("0.1"),
	    new ArrayBlockReader("O1", "OutputProjectionCirclePropertyRecord",
				 new String[][]{{"colorName", "red"},
						{"radius", "0.1"}
				 }, null)
		};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }
    
    private static BlockReader newEntity2Br() // creates an ArrayBlockReader defining an Entity
    {
	String header = "entity", parameter = "";
	String[][] entries = 
	{{"name", "Entity2"},
	 {"position", "(0, 0, 0)"},
	 {"velocity", "(0,0.5,0.25)"},
	 {"inertialMass", "2"}
	};
	BlockReader[] subblocks = new BlockReader[]
	{
	    newSphereCollPropBr("0.2"),
	    new ArrayBlockReader("O1", "OutputProjectionCirclePropertyRecord",
				 new String[][]{{"colorRGB", "(0,0,1)"},
						{"radius", "0.2"}
				 }, null)
		};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }
    private static BlockReader newEntity3Br() // creates an ArrayBlockReader defining an Entity
    {
	String header = "entity", parameter = "";
	String[][] entries = 
	{{"name", "Entity3"},
	 {"position", "(-1, 0, 0)"},
	 {"velocity", "(0,0,0)"},
	 {"inertialMass", "1"}
	};
	BlockReader[] subblocks = new BlockReader[]
	{
	    newSphereCollPropBr("0.2"),
	    new ArrayBlockReader("O1", "OutputProjectionCirclePropertyRecord",
				 new String[][]{{"colorName", "green"},
						{"radius", "0.2"}
				 }, null)
	};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    // *** ArrayBlockReader functions added by James Kermode
	
    // creates the entities {...} block containing all the entities as subblocks
    private static BlockReader newEntitiesBr(BlockReader[] subblocks)
    {
	String header = "entities", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    // creates the laws {...} block containing all the laws as subblocks
    private static BlockReader newLawsBr(BlockReader[] subblocks)
    {
	String header = "laws", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    // creates the physics {...} block containing laws {...} and entities{...} as subblocks
    private static BlockReader newPhysicsBr(BlockReader laws, BlockReader entities)
    {
	String header = "physics", parameter = null;
	String[][] entries = null;
	BlockReader[] subblocks = {laws, entities};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    /** Create a BlockReader object to construct a SphereCollision law from */
    private static BlockReader newSphereCollBr()
    {
	String header = "law", parameter = "SphereCollision"; 
	String[][] entries = 
	{{"name", "SphereCollision"}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** creates an ArrayBlockReader defining a SphereCollisionPropertyRecord */
    private static BlockReader newSphereCollPropBr(String radius) 
    {
	String header = "SphereCollision", parameter = null; 
	String[][] entries = {
	    {"radius", radius}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** creates an ArrayBlockReader defining a OutputProjection law */
    private static BlockReader newOutputProjectionBr(String name, String period, String normal, 
						     String xmin,
						     String xmax, String ymin, String ymax,
						     String width, String height,
						     String gifName)
    {
	String header = "law", parameter = "noope.output.OutputProjection";
	String[][] entries =
	{{"name", name},
	 {"period", period},
	 {"normal", normal},
	 {"xmin", xmin},
	 {"xmax", xmax},
	 {"ymin", ymin},
	 {"ymax", ymax},
	 {"width", width},
	 {"height", height},
	 {"gifName", gifName}
	};
	BlockReader[] subblocks = {};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }
}
