/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.awt.*;
import java.awt.event.*;

import java.lang.IllegalArgumentException;

import noope.output.MessageBox;

/**
 * Title:        AWT Gravity and Sphere Collision Demo Applet
 * Description:  This applet demonstrates the use of NOOPE to simulate the Gravity and SphereCollision laws. It uses AWT, so it can be run in old browsers.
 * 
 * @author Miklos Reiter
 * @version May 2001
 */

public class EditEntityDialog extends Dialog {

    /** Stores the owner; getOwner() apparently didn't exist in Java 1.1. */
    protected Frame owner;

    /**
     * Whether the dialog has been cancelled.
     */
    protected boolean cancelled;

    /**
     * Stores the data entered by the user. This is set when the user presses the OK button and
     * it is read by the static method that displayed the dialog.
     */
    protected EntityBrData result;

  Panel panelMain = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  Panel panelButtons = new Panel();
  Button buttonCancel = new Button();
  Button buttonOK = new Button();
  Panel panelEditorComponents = new Panel();
  Label lblName = new Label();
  TextField txtName = new TextField();
  Label lblm = new Label();
  TextField txtm = new TextField();
  Label lblr = new Label();
  TextField txtr = new TextField();
  Label lblx = new Label();
  TextField txtx = new TextField();
  Label lblv = new Label();
  TextField txtv = new TextField();
  Label lblColor = new Label();
  TextField txtColor = new TextField();

  protected EditEntityDialog(Frame owner, String title) {
    super(owner, title, true);
    this.owner = owner;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    panelMain.setLayout(borderLayout1);
    buttonCancel.setLabel("Cancel");
    buttonCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonCancel_actionPerformed(e);
      }
    });
    buttonOK.setLabel("OK");
    buttonOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonOK_actionPerformed(e);
      }
    });
    lblName.setText("Name: ");
    txtName.setColumns(5);
    lblm.setText(" m = ");
    txtm.setColumns(5);
    lblr.setText(" r = ");
    txtr.setColumns(5);
    lblx.setText(" x = ");
    txtx.setColumns(5);
    lblv.setText(" v = ");
    txtv.setColumns(5);
    lblColor.setText(" color = ");
    txtColor.setColumns(5);
    panelMain.add(panelButtons, BorderLayout.SOUTH);
    panelButtons.add(buttonCancel, null);
    panelButtons.add(buttonOK, null);
    panelMain.add(panelEditorComponents, BorderLayout.CENTER);
    panelEditorComponents.add(lblName, null);
    panelEditorComponents.add(txtName, null);
    panelEditorComponents.add(lblm, null);
    panelEditorComponents.add(txtm, null);
    panelEditorComponents.add(lblr, null);
    panelEditorComponents.add(txtr, null);
    panelEditorComponents.add(lblx, null);
    panelEditorComponents.add(txtx, null);
    panelEditorComponents.add(lblv, null);
    panelEditorComponents.add(txtv, null);
    panelEditorComponents.add(lblColor, null);
    panelEditorComponents.add(txtColor, null);
    this.add(panelMain);
    this.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        buttonCancel_actionPerformed(null);
      }
    });

    this.pack();
  }

    /**
     * Displays a new modal EditEntityDialog allowing the user to enter
     * details for an entity.
     */
    public static EntityBrData newEntityDialog(Frame owner)
    {
	EditEntityDialog d = new EditEntityDialog(owner, "New Entity");
	d.setVisible(true);
	if (d.cancelled) return null;
	return d.result;
    }
    
    
    /**
     * Displays a new modal EditEntityDialog allowing the user to edit data. If the user
     * cancels the dialog, this method returns null.
     */
    public static EntityBrData editEntityDialog(Frame owner, EntityBrData data)
    {
	if (data == null) return newEntityDialog(owner);
	EditEntityDialog d = new EditEntityDialog(owner, "Edit Entity");
	// set up data
	d.txtName.setText(data.getName());
	d.txtm.setText(data.getMass());
	d.txtr.setText(data.getRadius());
	d.txtx.setText(data.getPosition());
	d.txtv.setText(data.getVelocity());
	d.txtColor.setText(data.getColor());
	d.setVisible(true);
	if (d.cancelled) return null;
	return d.result;
    }

    /**
     * This method is called when the user presses the OK button. Creates a new EntityBrData object
     * and stores it in result. If the user has not entered all the data or the data is not
     * properly formatted, a dialog box is displayed informing the user, and the focus is set
     * back to the TextField causing the trouble. If all the data is valid, cancelled is set to
     * false and the dialog is hidden.
     */
    protected void buttonOK_actionPerformed(ActionEvent e) 
    {
	// get all the data
	result = new EntityBrData();
	
	result.setName(txtName.getText());

	try
	    {
		result.setMass(txtm.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid double for the mass.");
		txtm.requestFocus();
		return;
	    }

	try
	    {
		result.setRadius(txtr.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid double for the radius.");
		txtr.requestFocus();
		return;
	    }

	try
	    {
		result.setPosition(txtx.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Vector3D for the position.");
		txtx.requestFocus();
		return;
	    }

	try
	    {
		result.setVelocity(txtv.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Vector3D for the velocity.");
		txtv.requestFocus();
		return;
	    }

	try
	    {
		result.setColor(txtColor.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid color (either a predefined name " +
					  "or a Vector3D with components 0.0-1.0).");
		txtColor.requestFocus();
		return;
	    }
	
	// return from the modal dialog
	cancelled = false;
	this.setVisible(false);
    }

    /** Called when the user presses the Cancel button or closes the dialog. Sets cancelled to
     *	true and hides the dialog.
     */
    protected void buttonCancel_actionPerformed(ActionEvent e)
    {
	cancelled = true;
	this.setVisible(false);
    }
}
