/**
 * Copyright (C) 2001  Paul Evans (see the file README for details on how
 * to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.core;

import java.text.ParseException; // thrown by Vector3D(String)

// import noope.core.Vector3D;

/**
 * A mutable extension of Vector3D.
 *
 * @author Paul Evans
 */
public class MutableVector3D extends Vector3D
{

    /** Construct a MutableVector3D and initialise it to (0, 0, 0). */
    public MutableVector3D()
    {
	super();
    }
    
    /** Construct a MutableVector3D and initialise it to (new_x, new_y, new_z).
     * @param new_x x component
     * @param new_y y component
     * @param new_z z component
     */
    public MutableVector3D(double new_x, double new_y, double new_z)
    {
	super(new_x, new_y, new_z);
    }
    
    /** Construct a MutableVector3D equal to <i>v</i>.
     * @param v the vector to copy from
     */
    public MutableVector3D(Vector3D v)
    {
	super(v);
    }
    
    /** Construct a new MutableVector3D by parsing a string.
     * Parses the String <i>s</i> for a Vector3D given in the form (x, y, z) where
     * <i>x</i>, <i>y</i> and <i>z</i> are doubles. All whitespace is stripped and "0" is recognised
     * to mean the zero vector (0, 0, 0).
     * @param s the string to parse
     * @throws ParseException if there was an error parsing the string
     */
    public MutableVector3D(String s) throws ParseException
    {
	super(s);
    }

    /** Set the x component of this Vector3D
     * @param new_x x component
     */
    public void setX(double new_x)
    {
        x = new_x;
    }
    
    /** Set the y component of this Vector3D
     * @param new_y y component
     */
    public void setY(double new_y)
    {
        y = new_y;
    }
    
    /** Set the z component of this Vector3D
     * @param new_z z component
     */
    public void setZ(double new_z)
    {
        z = new_z;
    }
    
    /** Multiply each component of this Vector3D by the scalar <i>d</i>
      inplace. */
    public void multScalarInplace(double d)
    {
	x *= d;
	y *= d;
	z *= d;
    }

    /** Increment this Vector3D by the Vector3D <i>v</i> inplace. */
    public void addInplace(Vector3D v)
    {
	x += v.x;
	y += v.y;
	z += v.z;
    }

   /** Decrement this Vector3D by the Vector3D <i>v</i> inplace. */
    public void subtractInplace(Vector3D v)
    {
	x -= v.x;
	y -= v.y;
	z -= v.z;
    }
}
