/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


package noope.input;

import java.lang.Exception;

/** An exception of this class (or one of its subclasses) is thrown by an object failing to load
    itself from a BlockReader. It stores a BlockReaderLocation object referencing the position where
    the error occurred.
    @author Miklos Reiter
    @version 26 March 2001, original version
*/
public class BRLoadingException extends Exception
{
    private BlockReaderLocation location; // the position where the exception has been raised

    /** Constructs a new BRLoadingException object.
	@param message The error message.
	@param where The position in the BlockReader where the exception has occurred.*/
    public BRLoadingException(String message, BlockReaderLocation where)
    {
	super(message);
	location = where;
    }

    
    /** Returns a BlockReaderLocation object referring to the position in the BlockReader at which
	the exception has been raised. */
    public BlockReaderLocation getLocation()
    {
	return location;
    }
}
