/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.entities.Entity;
import noope.core.Physics;

import java.lang.Object;
import java.lang.Class;
import java.lang.reflect.Constructor;

/**
 *
 * This EntityLoader class loads Entity-s dynamically.
 *
 *
 */
public class DynamicEntityLoader extends EntityLoader
{
    /**
     * Stores the default package used if the class cannot be found with its exact name.
     */
    protected String defaultPackage;

    // List of formal parameters needed:
    private Class[] paramsClasses;

    /**
     * Constructs a new DynamicEntityLoader that first tries to load an Entity class with exactly
     * the name provided, and, if that fails, tries to look in the package specified.
     */
    public DynamicEntityLoader(String aDefaultPackage) throws BRLoadingException
    {
	defaultPackage = aDefaultPackage;
	paramsClasses = new Class[]{BRClassLoaderUtils.getClassWithExactName
				    (null, "noope.input.BlockReader"),
				    BRClassLoaderUtils.getClassWithExactName
				    (null, "noope.core.Physics")};
    }

    /**
     * Dynamically loads the Entity class with the given name and creates an instance of it.
     */
    public Entity createEntity(String className, BlockReaderLocation brL, 
			       BlockReader br, Physics phys)
	throws BRLoadingException
    {

	Object[] params = {br, phys};

	// Dynamically load the class
	Class c = BRClassLoaderUtils.getClassWithName(brL, className, defaultPackage);
 	    
	// assert it's a subclass of Entity
	BRClassLoaderUtils.assertSubClass
	    (brL, BRClassLoaderUtils.getClassWithExactName(null, "noope.entities.Entity"), c);
	
	// Construct an instance of the class and add it to the Entities list
	Constructor cons = BRClassLoaderUtils.getConstructor(brL, c, paramsClasses);
	
	return (Entity)(BRClassLoaderUtils.getNewInstance(brL, c, cons, params));
	
    }
}
