/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.entities.Entity;
import noope.laws.LawPropertyRecord;

import java.lang.Class;
import java.lang.reflect.Constructor;

/**
 * This is a LawPropertyRecordLoader that dynamically loads the class with the given name. This
 * is what is used by noope by default.
 *
 * @author Miklos Reiter
 * @version 29-Apr-2001
 *
 */
public class DynamicLawPropertyRecordLoader extends LawPropertyRecordLoader
{
    /**
     * The default package to load classes from if they cannot be found with the exact name.
     *
     */
    protected String defaultPackage;

    /**
     * Constructs a new DynamicLawPropertyRecordLoader object.
     *
     * @arg aDefaultPackage The default package to load things from if they cannot be found with
     * the exact name.
     */
    public DynamicLawPropertyRecordLoader(String aDefaultPackage)
    {
	defaultPackage = aDefaultPackage;
    }

    /**
     * This method dynamically loads the class with the given name.
     * The arguments, except for className and brL 
     * are passed
     * on to the constructor, so the class has to have a constructor of this form.
     *
     * @arg className The name of the class to load.
     * @arg br This is the BlockReader to use to construct the instance.
     * @arg brL The BlockReaderLocation used to construct a BRLoadingException if the creation
     * of the instance fails.
     * @arg ent The Entity for which the LawPropertyRecord is going to be constructed.
     */
    public LawPropertyRecord createLawPropertyRecord(String className, 
							      BlockReaderLocation brL, 
							      BlockReader br, Entity ent)
	throws BRLoadingException
    {
	Class theClass = BRClassLoaderUtils.getClassWithName(brL, className, defaultPackage);
	
	// assert it's a subclass
	// of LawPropertyRecord
	BRClassLoaderUtils.assertSubClass
	    (brL, BRClassLoaderUtils.getClassWithExactName
	     (null, "noope.laws.LawPropertyRecord"), theClass);
	
	// now let's make an instance of it:
	// List of formal parameters needed:
	Class[] paramsClasses = {BRClassLoaderUtils.getClassWithExactName(null, 
									  "noope.input.BlockReader"),
				 BRClassLoaderUtils.getClassWithExactName(null, 
									  "noope.entities.Entity")};
	Object[] params = {br, ent};
	
	// get constructor
	Constructor theConstructor = 
	    BRClassLoaderUtils.getConstructor(brL, theClass, paramsClasses);
	
	// now let's use the constructor to make a new instance
	return (LawPropertyRecord)BRClassLoaderUtils.getNewInstance
	    (brL, theClass, theConstructor, params);
    }
}
