/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.core.Physics;
import noope.laws.Law;

/**
 * This is an implementation of LawLoader that hard-codes all the known Law classes.
 *
 * This is meant as a workaround for Netscape's paranoid SecurityManager. In applications, use
 * DynamicLawLoader.
 *
 * @author Miklos Reiter
 * @version 28-Apr-2001
 *
 */
public class HardCodedLawLoader extends LawLoader
{
    /**
     * This constructor does nothing.
     */
    public HardCodedLawLoader()
    {
    }

    /**
     * Returns a new Law object.
     *
     * This class knows the non-abstract laws as of 29-Apr-2001.
     *
     * @arg className The name of the class to load.
     * @arg br This is the BlockReader to use to construct the law.
     * @arg brL The BlockReaderLocation used to construct a BRLoadingException if the creation
     * of the Law fails.
     * @arg lawnumber This is the number of the law to use.
     * @arg phys This is a reference to the Physics object that is being constructed and to
     * which this law will belong.
     */
    public Law createLaw(String className, BlockReaderLocation brL, BlockReader br, 
				  int lawnumber, Physics phys)
	throws BRLoadingException
    {
	if (className.equals("OutputText"))
	    return new noope.output.OutputText(br, lawnumber, phys);
	if (className.equals("OutputProjection"))
	    return new noope.output.OutputProjection(br, lawnumber, phys);
	if (className.equals("Gravity"))
	    return new noope.laws.Gravity(br, lawnumber, phys);
	if (className.equals("SphereCollision"))
	    return new noope.laws.SphereCollision(br, lawnumber, phys);

	// now do packaged versions:
	if (className.equals("noope.output.OutputText"))
	    return new noope.output.OutputText(br, lawnumber, phys);
	if (className.equals("noope.output.OutputProjection"))
	    return new noope.output.OutputProjection(br, lawnumber, phys);
	if (className.equals("noope.laws.Gravity"))
	    return new noope.laws.Gravity(br, lawnumber, phys);
	if (className.equals("noope.laws.SphereCollision"))
	    return new noope.laws.SphereCollision(br, lawnumber, phys);

	// not found? Complain!
	throw new BRLoadingException("The law class " + className + " is unknown to this "
				     + "HardCodedLawLoader!", brL);
    }
}
