/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.laws;

import noope.core.*;
import noope.input.BlockReader;
import noope.entities.Entity;

/** This is a class that stores the properties of an Entity needed by a 
specific Law. Each Entity stores a LawPropertyRecord for every Law present 
in Laws. This class is <b>not</b> abstract, but it does not store any data, so 
for most Laws, it will be desirable to derive subclasses.

Every subclass should have constructor following the same interface. 
This is not a requirement, since only the Law whose LawPropertyRecord 
this is is ever going to construct one, but it is a good idea for consistent
style.

@author Miklos Reiter
@version 19 April 2001: added dispose() [Miklos Reiter]
*/
public class LawPropertyRecord
{
    /** Construct a LawPropertyRecord from the supplied BlockReader,
       for the supplied Entity. This method does nothing, so it should
       be overridden in subclasses. Every subclass 
       should have constructor following this interface. 
       @param br BlockReader used to initialise LawPropertyRecord
       @param ent Entity for which the LawPropertyRecord is being constructed */
    public LawPropertyRecord(BlockReader br, Entity ent)
    { // do nothing
    }

    /** Disposes of system resources held by this LawPropertyRecord. Does nothing if not
	overridden. */
    public void dispose()
    {

    }
}
