/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.laws;

import noope.input.BlockReader;
import noope.input.BlockReaderLocation;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;
import noope.entities.Entity;

/** This is the LawPropertyRecord used by the SphereCollision law to store the radius of each
 *  Entity.
 *
 * @author Miklos Reiter
 * @version June 2001
 *
 */
public class SphereCollisionPropertyRecord extends LawPropertyRecord
{
    private double radius; // stores the radius of this Entity.

    /** Constructs a new SphereCollisionPropertyRecord from the given BlockReader. This tries to
	find the entry with the key "radius". If it fails to do so, or the value cannot be
	interpreted as a double, an exception is raised. */
    public SphereCollisionPropertyRecord(BlockReader br, Entity ent) throws BRLoadingException
    {
	super(br, ent); // call the inherited constructor
	
	String[] brEntry = null; // store an entry from the BlockReader here temporarily

        // just peek first, for what if we are not supposed to be reading this...

	if (br == null) throw new BRLoadingException("Cannot construct " +
						     "SphereCollisionPropertyRecord from " +
						     "empty BlockReader.", null);

        if (br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("radius")))
            {
                try
                    {
                        radius = Double.valueOf(brEntry[1]).doubleValue();
                        br.getNextEntry(); // if we need the entry, advance pointer in br
                    }
                catch (NumberFormatException e)
                    {
			BlockReaderLocation loc = br.getEntryLocation();
			BRLoadingParseException error = new 
			    BRLoadingParseException("Failed to parse '" + brEntry[1]
							     + "' for double.", loc);
			throw(error);
                    }
            }
        else // complain
            {
                BlockReaderLocation loc = br.getEntryLocation();
		BREntryNotFoundException error = new
		    BREntryNotFoundException("radius", loc);
		throw(error);
            }
	
    }

    /** Returns the radius of this (spherical) entity. */
    public double getRadius()
    {
	return radius;
    }
}
