/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.awt.Graphics2D;

/**
 *
 * The OutputProjection Law uses an object implementing this interface for displaying
 * animations, or outputting
 * them to a gif file... . OutputProjection draws into the AnimationListener by drawing to the
 * Graphics2D object returned by drawingGraphics().
 *
 * @author Miklos Reiter
 * @version 05-May-2001
 */
public interface AnimationListener
{
    /** Returns a Graphics2D object that can be used to draw into the BufferedImage.
     */
    public Graphics2D drawingGraphics();

    /** Should be called when  */
    public void updateComplete();

    /** Whether this AnimationListener is ready to be sent image information (or whether it is
	useful to do so. For example, a window, after being made invisible does not need any
	more graphics information.) Returns false after dispose() has been called.*/
    public boolean isWorking();

    /** Disposes of system resources used and finished processing, e.g. closing open files... .*/
    public void dispose();
}
