/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;


import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;

import noope.entities.Entity;

import java.lang.Double;

/** This subclass of OutputProjectionCirclePropertyRecord represents the Entity as a circle.
    Caution: if the resolution in the x and y directions is not the same, the circle will look
    like an ellipse.

    @author Miklos Reiter
    @version 19 April 2001, bug fixed: now moveShTo works correctly
*/
public class OutputProjectionCirclePropertyRecord extends OutputProjectionShapePropertyRecord
{

    /** Stores the radius of the circle. */
    protected double radius;	

    /** Calls the inherited constructor and then reads the double "radius" from br. If it fails, 
	it throws an exception.
    */
    public OutputProjectionCirclePropertyRecord(BlockReader br, Entity ent) throws
	BRLoadingException
    {
	super(br, ent);
	
	String[] brEntry;
	
	if (br == null)
	    throw new BRLoadingException("Cannot construct a OutputProjectionCirclePropertyRecord"
					 +" from a null BlockReader", null);

	// get radius, will not necessarily be read from br, so use peek...
	if (br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("radius")))
        {
            try
	    {
		br.getNextEntry(); // advance Entry pointer
		radius = Double.parseDouble(brEntry[1]);
            }
            catch (NumberFormatException e)
            {
		throw (new
		    BRLoadingParseException("Failed to parse for double" , br.getEntryLocation()));
            }
	}
        else
        {
	    throw new BREntryNotFoundException("radius", br.getEntryLocation());
        }

	// construct "circle", i.e. ellipse
	sh = new java.awt.geom.Ellipse2D.Double(0.0, 0.0, 2*radius, 2*radius);
    }
    
    /** Moves sh to (x, y). I.e. after the call, the centre of sh will be in (x, y). */
    protected void moveShTo(double x_pos, double y_pos)
    {
	java.awt.geom.Ellipse2D.Double ell = (java.awt.geom.Ellipse2D.Double)sh;
	ell.x = x_pos - (0.5*ell.width);
	ell.y = y_pos - (0.5*ell.height);
	/*	((java.awt.geom.Ellipse2D.Double)sh).x = x_pos;
		((java.awt.geom.Ellipse2D.Double)sh).y = y_pos;*/
    }
    
}
