/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

// noope packages
import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;

import noope.entities.Entity;

// graphics packages
import java.awt.Graphics2D;
import java.awt.Shape;


/** This is an abstract OutputProjectionPropertyRecord class intended to be the base class for
    things that are to be represented by an object implementing java.awt.geom.Shape.

    @author Miklos Reiter
    @version 30 March, 2001
 */
public abstract class OutputProjectionShapePropertyRecord 
    extends OutputProjectionColoredPropertyRecord
{
    /** The default value for fill. */
    public static boolean defaultFill = true;

    /** Whether the shape should be filled when drawn. Otherwise, only the contours are drawn.*/
    protected boolean fill;

    /** Stores the Shape object to draw. */
    protected Shape sh;

    /** Constructs a new OutputProjectionShapePropertyRecord object from the BlockReader br, for
	the Enitity ent. Apart from the data loaded from br by the inherited constructor, we als
	get a boolean (i.e. true/false/True/False/TRUE/FALSE/1/0) fill that specifies whether
	to fill the inside of the
	shape or not. The default is defaultFill.
    */
    public OutputProjectionShapePropertyRecord(BlockReader br, Entity ent)
	throws BRLoadingException
    {
	super(br, ent);

	String[] brEntry;

	// get fill (optional, so use peek...)
	if (br != null && br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("fill")))
	    {
		br.getNextEntry(); // advance pointer
		if (brEntry[1].equals("true") || brEntry[1].equals("True") || brEntry[1].equals("TRUE")
		    || brEntry[1].equals("1")) fill = true;
		else
		    if (brEntry[1].equals("false") || brEntry[1].equals("False") || 
			brEntry[1].equals("FALSE") || brEntry[1].equals("0")) fill = false;
		    else
			throw new BRLoadingParseException("Failed to parse for boolean. "
							  +"Expected one of the values: "
							  +"true/flase/True/False/TRUE/"
							  +"FALSE/1/0.",
							  br.getEntryLocation());
	    }
	else // don't complain if nothing is given, use default!
	    {
		fill = defaultFill;
	    }
    }

    /** Draws sh */
    public void draw(Graphics2D g, double X, double Y)
    {
	// display new point
	moveShTo(X, Y);
	g.setColor(color);
	if (fill) g.fill(sh);
	else g.draw(sh);
    }

    /** Moves sh to the point with coordinates (x, y). After this operation, we expect the *centre*
	of sh to be at (x, y). */
    abstract protected void moveShTo(double x, double y);
}
